package jwtmdl

import (
	"fmt"
	"net/http"
	"testing"
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/authmdl/sessionmdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/cachemdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/utiliymdl/guidmdl"
	"github.com/gin-gonic/gin"
	jwt "github.com/golang-jwt/jwt/v5"
	"github.com/stretchr/testify/assert"
	"github.com/tidwall/gjson"
)

const (
	TestKey = "vJUufKHyu2xiMYmDj1TmojHR11ciUaq3"
)

func checkToken(token string) error {
	claims, err := decode(jwt.Parse(token, keyFunc(TestKey)))
	if err != nil {
		return err
	}

	return sessionmdl.ValidateSessionFromToken(claims)
}

func server() {
	g := gin.Default()
	g.GET("/status", func(c *gin.Context) {
		fmt.Println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ ")

		c.JSON(200, "Hello")
	})
	g.Run("localhost:8081")
}
func TestDecodeTokenSuccess(t *testing.T) {
	// go server()
	// fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>calling rq ")
	// resp, err := http.NewRequest("GET", "http://localhost:8081/status", nil)
	// fmt.Println("resp  ", resp)
	// fmt.Println("err  ", err)

	//time.Sleep(30 * time.Second)

	// got, _ := DecodeToken("Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJJZCI6InNhbmRlZXBzcyIsImdyb3VwcyI6ImFkbWluIn0.qmA-brbKScqBEFr3wJIuBmcBsERYoNPLnOAHBe4HNh8", "3212mysupersecretpassword")
	// assert.Equal(t, "sandeepss", got["Id"], "it is equal")

}

func TestDecodeTokenvalid(t *testing.T) {
	req, _ := http.NewRequest("POST", "/r/c/mql", nil)
	req.Header.Set("Authorization", "Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJJZCI6InNhbmRlZXBzcyIsImdyb3VwIjpbImFkbWluIl19.dGSKQZMGSEBjgPrjTHJwylcxIxxMKylahuFn7DHR4pc")
	_, derror := DecodeTokenWithJWTKey(req, "mysupersecretpassword")
	assert.NoError(t, derror, "No error occured")

}

// func TestDecodeTokenInvalid(t *testing.T) {
// 	req, _ := http.NewRequest("POST", "/r/c/mql", nil)
// 	req.Header.Set("Authorization", "Bearer yJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJJZCI6InNhbmRlZXBzcyIsImdyb3VwcyI6ImFkbWluIn0.qmA-brbKScqBEFr3wJIuBmcBsERYoNPLnOAHBe4HNh8")
// 	_, derror := DecodeToken(req, "mysupersecretpassword")
// 	assert.Error(t, derror, "error occured")

// }
// func TestDecodeTokenEmpty(t *testing.T) {
// 	req, _ := http.NewRequest("POST", "/r/c/mql", nil)
// 	req.Header.Set("Authorization", "")
// 	_, derror := DecodeToken(req, "mysupersecretpassword")
// 	assert.Error(t, derror, "error occured")

// }

// func TestDecodeTokenWrongAlo(t *testing.T) {
// 	errormdl.IsTestingNegetiveCaseOnCheckBool = true
// 	req, _ := http.NewRequest("POST", "/r/c/mql", nil)
// 	req.Header.Set("Authorization", "Bearer yJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJJZCI6InNhbmRlZXBzcyIsImdyb3VwcyI6ImFkbWluIn0.qmA-brbKScqBEFr3wJIuBmcBsERYoNPLnOAHBe4HNh8")
// 	_, derror := DecodeToken(req, "mysupersecretpassword")
// 	assert.Error(t, derror, "error occured")
// 	errormdl.IsTestingNegetiveCaseOnCheckBool = false
// }
// func TestDecodeTokenWrongTypeConversion(t *testing.T) {
// 	errormdl.IsTestingNegetiveCaseOnCheckBool1 = true
// 	req, _ := http.NewRequest("POST", "/r/c/mql", nil)
// 	req.Header.Set("Authorization", "Bearer yJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJJZCI6InNhbmRlZXBzcyIsImdyb3VwcyI6ImFkbWluIn0.qmA-brbKScqBEFr3wJIuBmcBsERYoNPLnOAHBe4HNh8")
// 	_, derror := DecodeToken(req, "mysupersecretpassword")
// 	assert.Error(t, derror, "error occured")
// 	errormdl.IsTestingNegetiveCaseOnCheckBool1 = false
// }

func TestGenerateTokenWithOptions(t *testing.T) {
	sessionmdl.InitUserSessionCache(cachemdl.TypeFastCache)

	type args struct {
		args []Option
	}
	tests := []struct {
		name    string
		args    args
		want    string
		wantErr bool
	}{
		{
			name: "Token without session",
			args: args{
				args: []Option{
					WithUserID("tom@company.org"),
					WithExpiration(0),
					WithKey(TestKey),
					WithMetaData(`{"name":"tom"}`),
				},
			},
			wantErr: false,
		},
		{
			name: "Token with session",
			args: args{
				args: []Option{
					WithUserID("tom@company.org"),
					WithExpiration(0),
					WithKey(TestKey),
					WithMetaData(`{"name":"tom"}`),
					WithSession(guidmdl.GetGUID(), "me"),
				},
			},
			wantErr: false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := GenerateTokenWithOptions(tt.args.args...)
			if (err != nil) != tt.wantErr {
				t.Errorf("GenerateTokenWithOptions() error = %v, wantErr %v", err, tt.wantErr)
				return
			}

			err = checkToken(got)
			if (err != nil) != tt.wantErr {
				t.Errorf("GenerateTokenWithOptions() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
		})
	}
}

func TestGenerateTokenWithJWTKey(t *testing.T) {
	type args struct {
		loginID        string
		groups         []string
		clientIP       string
		metadata       gjson.Result
		expirationTime time.Duration
		JWTKey         string
	}
	tests := []struct {
		name string
		args args
		// want    string
		wantErr bool
	}{
		{
			name: "Test genrate token",
			args: args{
				JWTKey:         TestKey,
				expirationTime: time.Minute * 5,
				groups:         []string{"admin"},
				loginID:        "tom@company.org",
				metadata:       gjson.Parse(`{"name":"tom"}`),
			},
			wantErr: false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := GenerateTokenWithJWTKey(tt.args.loginID, tt.args.groups, tt.args.clientIP, tt.args.metadata, tt.args.expirationTime, tt.args.JWTKey)
			if (err != nil) != tt.wantErr {
				t.Errorf("GenerateTokenWithJWTKey() error = %v, wantErr %v", err, tt.wantErr)
				return
			}

			err = checkToken(got)
			if (err != nil) != tt.wantErr {
				t.Errorf("GenerateTokenWithJWTKey() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
		})
	}
}
