package ldapmdl

import (
	"testing"

	"github.com/stretchr/testify/assert"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
)

func TestLDAPAuthentication(t *testing.T) {

	//LDAP init config error
	errormdl.IsTestingNegetiveCaseOn = true
	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")
	errormdl.IsTestingNegetiveCaseOn = false

	//LDAP BaseDSN not set
	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")
	ldapConfig.BaseDN = ""
	authStatusCode, _ := AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//LDAP FilterDN not set
	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")
	ldapConfig.FilterDN = ""
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//LDAP LDAPPassword not set
	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")
	ldapConfig.LDAPPassword = ""
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//LDAP LDAPServerIPWithPort not set
	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")
	ldapConfig.LDAPServerIPWithPort = ""
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//LDAP LDAPUsername not set
	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")
	ldapConfig.LDAPUsername = ""
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	InitLDAP("../../testingdata/testData/ldapmdl/config.toml")

	//LDAP connection error
	errormdl.IsTestingNegetiveCaseOn = true
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//TLS connection establishment error
	errormdl.IsTestingNegetiveCaseOn = false
	errormdl.IsTestingNegetiveCaseOn1 = true
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//Administrative user bind error
	errormdl.IsTestingNegetiveCaseOn = false
	errormdl.IsTestingNegetiveCaseOn1 = false
	errormdl.IsTestingNegetiveCaseOn2 = true
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 0, authStatusCode)

	//AD user search error
	errormdl.IsTestingNegetiveCaseOn = false
	errormdl.IsTestingNegetiveCaseOn1 = false
	errormdl.IsTestingNegetiveCaseOn2 = false
	errormdl.IsTestingNegetiveCaseOn3 = true
	authStatusCode, _ = AuthenticateOnLDAP("rahuls123", "dfgh")
	assert.Equal(t, 0, authStatusCode)

	//Blank username
	authStatusCode, _ = AuthenticateOnLDAP("", "dfgh")
	assert.Equal(t, 0, authStatusCode)

	//Blank password
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "")
	assert.Equal(t, 0, authStatusCode)

	//User does not exist test
	errormdl.IsTestingNegetiveCaseOn = false
	errormdl.IsTestingNegetiveCaseOn1 = false
	errormdl.IsTestingNegetiveCaseOn2 = false
	errormdl.IsTestingNegetiveCaseOn3 = false
	authStatusCode, _ = AuthenticateOnLDAP("rahuls123", "dfgh")
	assert.Equal(t, 1, authStatusCode)

	//Mutiple user entries on AD server
	errormdl.IsTestingNegetiveCaseOnCheckBool = true
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 2, authStatusCode)

	//Wrong password
	errormdl.IsTestingNegetiveCaseOnCheckBool = false
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "fghjkl")
	assert.Equal(t, 3, authStatusCode)

	//Successful authentication
	//TODO: Enter correct username and password
	authStatusCode, _ = AuthenticateOnLDAP("rahuls", "aug#1234")
	assert.Equal(t, 4, authStatusCode)

}
