package roleenforcemdl

import (
	"encoding/csv"
	"os"
	"strconv"
	"strings"
	"sync"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	jwt "github.com/golang-jwt/jwt/v5"
)

var roleManagementConfig map[string]URLGroupMapping
var once sync.Once

// URLGroupMapping URLGroupMapping
type URLGroupMapping struct {
	All        bool
	Allowed    map[string]bool
	NotAllowed map[string]bool
}

func init() {
	roleManagementConfig = make(map[string]URLGroupMapping)
}

func LoadRoleConfiguration(csvFilePath string) {
	// Open CSV file
	once.Do(func() {
		f, err := os.Open(csvFilePath)
		if err != nil {
			panic(err)
		}
		defer f.Close()

		// Read File into a Variable
		lines, err := csv.NewReader(f).ReadAll()
		if err != nil {
			panic(err)
		}

		for _, line := range lines {
			group := strings.Trim(line[0], " ")
			URL := strings.Trim(line[1], " ")
			// method := strings.Trim(line[2], " ")

			boolValue := strings.Trim(line[3], " ")
			isAllowed, parseError := strconv.ParseBool(boolValue)
			if parseError != nil {
				loggermdl.LogError("error occured while parsing string to boolean")
			}
			registerUser(group, URL, isAllowed)
		}
	})
}

// RegisterUser - RegisterUser register use
func RegisterUser(group, URL string, isAllowed bool) {
	roleObject, ok := roleManagementConfig[URL]
	if ok {
		roleObject = assignURL(group, isAllowed, roleObject)
	} else {
		roleObject = URLGroupMapping{
			Allowed:    make(map[string]bool),
			NotAllowed: make(map[string]bool),
		}
		roleObject = assignURL(group, isAllowed, roleObject)
	}
	roleManagementConfig[URL] = roleObject
}

func registerUser(group, URL string, isAllowed bool) {
	roleObject, ok := roleManagementConfig[URL]
	if ok {
		roleObject = assignURL(group, isAllowed, roleObject)
	} else {
		roleObject = URLGroupMapping{
			Allowed:    make(map[string]bool),
			NotAllowed: make(map[string]bool),
		}
		roleObject = assignURL(group, isAllowed, roleObject)
	}
	roleManagementConfig[URL] = roleObject
}

func assignURL(group string, isAllowed bool, roleObj URLGroupMapping) URLGroupMapping {
	if group == "*" {
		roleObj.All = isAllowed
	} else {
		if isAllowed {
			roleObj.Allowed[group] = true
		} else {
			roleObj.NotAllowed[group] = true
		}
	}
	return roleObj
}

func checkAccess(URL string, groupList []string) bool {
	if roleObj, ok := roleManagementConfig[URL]; ok {
		if roleObj.All {
			for _, group := range groupList {
				if _, ok := roleObj.NotAllowed[group]; ok {
					return false
				}
			}
			return true
		}
		for _, group := range groupList {
			if _, ok := roleObj.Allowed[group]; ok {
				return true
			}
		}
	}
	return false
}

// GetGroupNames gets the group names from the request.
func GetGroupNames(decodedToken jwt.MapClaims, groupKey string) ([]string, error) {

	if decodedToken == nil {
		loggermdl.LogError("decodedToken is empty")
		return nil, errormdl.Wrap("Error while getting claims")
	}
	groups, ok := decodedToken[groupKey]

	if !ok {
		loggermdl.LogError("either group key is invalid or token does not contain the group data")
		return nil, errormdl.Wrap("either group key is invalid or token does not contain the group data")
	}

	eachGroup, ok := groups.([]interface{})
	if !ok {
		return nil, nil
	}
	stringArray := []string{}
	for _, str := range eachGroup {
		group, ok1 := str.(string)
		if !ok1 {
			return nil, errormdl.Wrap("Group not casted to string")
		}
		stringArray = append(stringArray, group)
	}

	return stringArray, nil
}
