// +build !fasthttp

package roleenforcemdl

import (
	"net/http"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/authmdl/jwtmdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
)

// Enforce Enforce
func Enforce(req *http.Request, URL, jwtKey, groupKey string) bool {
	groupList, geterr := GetGroupFromToken(req, jwtKey, groupKey)
	if errormdl.CheckErr(geterr) != nil {
		loggermdl.LogError("error occured while fetching groups from token : ", errormdl.CheckErr(geterr))
		return false
	}

	return checkAccess(URL, groupList)
}

func GetGroupFromToken(req *http.Request, jwtKey, groupKey string) ([]string, error) {
	claims, decodeError := jwtmdl.DecodeTokenWithJWTKey(req, jwtKey)
	if errormdl.CheckErr(decodeError) != nil {
		loggermdl.LogError("error occured while decoding jwt token : ", errormdl.CheckErr(decodeError))
		return nil, errormdl.CheckErr(decodeError)
	}
	groups, groupFetchError := GetGroupNames(claims, groupKey)
	if errormdl.CheckErr(groupFetchError) != nil {
		loggermdl.LogError("error occured while fetching groups from token : ", errormdl.CheckErr(groupFetchError))
		return nil, errormdl.CheckErr(groupFetchError)
	}
	return groups, nil
}
