package sessionmdl

import (
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/cachemdl"
)

func init() {
	Init(cachemdl.SetupFastCache(cachemdl.FCWithMaxEntries(10000)))
}

func TestSet(t *testing.T) {
	Set("tom@company.org", Session{SessionFor: "xyz", SessionId: "789"})

	type args struct {
		userId string
		s      []Session
	}
	tests := []struct {
		name string
		args args
	}{
		// TODO: Add test cases.
		{
			name: "User present",
			args: args{
				s:      []Session{Session{SessionFor: "abc", SessionId: "123"}},
				userId: "tom@company.org",
			},
		},
		{
			name: "User not present",
			args: args{
				s:      []Session{Session{SessionFor: "abc", SessionId: "123"}},
				userId: "ronny@company.org",
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			Set(tt.args.userId, tt.args.s...)
		})
	}

	all := store.GetItemsCount()
	if all != len(tests) {
		t.Errorf("expected %d users got %d", len(tests), all)
	}
}

func TestValidateSessionFromToken(t *testing.T) {
	Set("tom@company.org", Session{SessionFor: "xyz", SessionId: "789"})
	type args struct {
		claims map[string]interface{}
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{
			name:    "Session present",
			args:    args{claims: map[string]interface{}{"userId": "tom@company.org", "sessionId": "789"}},
			wantErr: false,
		},
		{
			name:    "Session not present",
			args:    args{claims: map[string]interface{}{"userId": "invalid@company.org", "sessionId": "123"}},
			wantErr: true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := ValidateSessionFromToken(tt.args.claims); (err != nil) != tt.wantErr {
				t.Errorf("ValidateSessionFromToken() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func TestCheckForSessionAvailability(t *testing.T) {
	Set("tom@company.org", Session{SessionFor: "xyz", SessionId: "789"})
	type args struct {
		userId     string
		sessionFor string
	}
	tests := []struct {
		name string
		args args
		want bool
	}{
		{
			name: "Session present",
			args: args{userId: "tom@company.org", sessionFor: "xyz"},
			want: true,
		},
		{
			name: "Session not present",
			args: args{userId: "tom@company.org", sessionFor: "someRandomID"},
			want: false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if res := CheckForSessionAvailability(tt.args.userId, tt.args.sessionFor); res != tt.want {
				t.Errorf("CheckForSessionAvailability() got = %v, want %v", res, tt.want)
			}
		})
	}
}

func TestDeleteSession(t *testing.T) {
	Set("TestDeleteSession", Session{SessionFor: "xyz", SessionId: "789"})
	Set("TestDeleteSession", Session{SessionFor: "abc", SessionId: "123"})
	type args struct {
		userId     string
		sessionFor string
	}
	tests := []struct {
		name string
		args args
	}{
		{
			name: "Delete existing session",
			args: args{userId: "TestDeleteSession", sessionFor: "xyz"},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			DeleteSession(tt.args.userId, tt.args.sessionFor)
			i, _ := store.Get(tt.args.userId)
			// if !ok {
			// 	t.Error("expected", tt.args.userId, "not to be deleted")
			// }

			sessions := i.([]Session)

			for _, s := range sessions {
				if s.SessionFor == tt.args.sessionFor {
					t.Error("expected", tt.args.sessionFor, "to be deleted")
				}
			}
		})
	}

}
