package boltdb

import (
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"github.com/boltdb/bolt"
	"github.com/stretchr/testify/assert"
)

const (
	bucketName  = "test"
	dbPathWrong = "C:/test/test.db"
)

func TestInitDB(t *testing.T) {
	_, err := InitDB("C:/test.db", bucketName)
	assert.NoError(t, err, "This should not return error")
}
func TestInitDBOpenDbError(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn = true
	_, err := InitDB(dbPathWrong, bucketName)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should return error")
}
func TestInitDBBktError(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn1 = true
	_, err := InitDB("C:/test1.db", bucketName)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should return error")
}
func TestInitDBBktError2(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn2 = true
	_, err := InitDB("C:/test111.db", bucketName)
	errormdl.IsTestingNegetiveCaseOn2 = false
	assert.Error(t, err, "This should return error")
}
func TestAddKey(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test2.db", bucketName)
	err2 := AddKey(dbIntsance, bucketName, "testKey", "testValue")
	assert.NoError(t, err2, "This should not return error")
}

func TestAddKeyDbnil(t *testing.T) {
	var dbIntsance *bolt.DB
	err2 := AddKey(dbIntsance, bucketName, "testKey", "testValue")
	assert.Error(t, err2, "This should return error")
}
func TestAddKeyError(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test22.db", bucketName)
	errormdl.IsTestingNegetiveCaseOn = true
	err2 := AddKey(dbIntsance, bucketName, "testKey", "testValue")
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err2, "This should return error")
}

func TestRemoveKey(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test3.db", bucketName)
	AddKey(dbIntsance, bucketName, "testKey", "testValue")
	err3 := RemoveKey(dbIntsance, bucketName, "testKey")
	assert.NoError(t, err3, "This should not return error")
}
func TestRemoveKeyDbNil(t *testing.T) {
	var dbIntsance *bolt.DB
	AddKey(dbIntsance, bucketName, "testKey", "testValue")
	err3 := RemoveKey(dbIntsance, "bktnil", "testKey")
	assert.Error(t, err3, "This should return error")
}
func TestRemoveKeyBktNil(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test31.db", bucketName)
	AddKey(dbIntsance, bucketName, "testKey", "testValue")
	err3 := RemoveKey(dbIntsance, "bktnil", "testKey")
	assert.Error(t, err3, "This should return error")
}
func TestGetKey(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test4.db", bucketName)
	AddKey(dbIntsance, bucketName, "testKey", "testValue")
	_, err := GetKey(dbIntsance, bucketName, "testKey")
	assert.NoError(t, err, "This should not return error")
}
func TestGetKeyDbNil(t *testing.T) {
	var dbIntsance *bolt.DB
	AddKey(dbIntsance, bucketName, "testKey", "testValue")
	_, err := GetKey(dbIntsance, bucketName, "testKey")
	assert.Error(t, err, "This should return error")
}

func TestGetKeyBktErr(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test5.db", bucketName)
	_, err := GetKey(dbIntsance, "mdasda", "testKey")
	assert.Error(t, err, "This should return error")
}

type BucketData struct {
	Name string `json:"name"`
}

func TestAddRecord(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test6.db", bucketName)
	bucketData := BucketData{}
	bucketData.Name = "test"
	err := AddRecord(dbIntsance, bucketName, "myname", bucketData)
	assert.NoError(t, err, "This should not return error")
}
func TestAddRecordDbNil(t *testing.T) {
	var dbIntsance *bolt.DB
	bucketData := BucketData{}
	bucketData.Name = "test"
	err := AddRecord(dbIntsance, bucketName, "myname", bucketData)
	assert.Error(t, err, "This should return error")
}

func TestAddRecordError(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test7.db", bucketName)
	bucketData := BucketData{}
	bucketData.Name = "test"
	errormdl.IsTestingNegetiveCaseOn = true
	err := AddRecord(dbIntsance, bucketName, "myname", bucketData)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should return error")
}
func TestAddRecordError1(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test8.db", bucketName)
	bucketData := BucketData{}
	bucketData.Name = "test"
	errormdl.IsTestingNegetiveCaseOn1 = true
	err := AddRecord(dbIntsance, bucketName, "myname", bucketData)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should return error")
}
func TestAddRecordError2(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test81.db", bucketName)
	bucketData := BucketData{}
	bucketData.Name = "test"
	errormdl.IsTestingNegetiveCaseOn2 = true
	err := AddRecord(dbIntsance, bucketName, "myname", bucketData)
	errormdl.IsTestingNegetiveCaseOn2 = false
	assert.Error(t, err, "This should return error")
}
func TestGetRecord(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test9.db", bucketName)
	bucketData := BucketData{}
	bucketData.Name = "test"
	AddRecord(dbIntsance, bucketName, "myname", bucketData)
	_, err2 := GetRecord(dbIntsance, bucketName)
	assert.NoError(t, err2, "This should not return error")
}
func TestGetRecordDbNil(t *testing.T) {
	var dbIntsance *bolt.DB
	bucketData := BucketData{}
	bucketData.Name = "test"
	AddRecord(dbIntsance, bucketName, "myname", bucketData)
	_, err2 := GetRecord(dbIntsance, bucketName)
	assert.Error(t, err2, "This should return error")
}
func TestGetRecordBktNil(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test91.db", bucketName)
	bucketData := BucketData{}
	bucketData.Name = "test"
	AddRecord(dbIntsance, "nilBkt", "myname", bucketData)
	_, err2 := GetRecord(dbIntsance, "bktNil")
	assert.Error(t, err2, "This should return error")
}

func TestGetRecordError(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test10.db", bucketName)
	errormdl.IsTestingNegetiveCaseOn = true
	_, err2 := GetRecord(dbIntsance, bucketName)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.NoError(t, err2, "This should not return error")
}

func TestGetRecordError1(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test11.db", bucketName)
	errormdl.IsTestingNegetiveCaseOn1 = true
	_, err2 := GetRecord(dbIntsance, bucketName)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err2, "This should return error")
}

func TestGetRecordError2(t *testing.T) {
	dbIntsance, _ := InitDB("C:/test12.db", bucketName)
	errormdl.IsTestingNegetiveCaseOn2 = true
	_, err2 := GetRecord(dbIntsance, bucketName)
	errormdl.IsTestingNegetiveCaseOn2 = false
	assert.Error(t, err2, "This should return error")
}
