package bucket

import (
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/corefdb/filetype"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/corefdb/locker"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/utiliymdl/guidmdl"
	"github.com/tidwall/gjson"
)

type AppendBucket struct {
	Bucket
}

func NewAppendBucket(bucketNameQuery string, isDynamicName bool, isLazyEnable bool, bucketPath string) (*AppendBucket, error) {
	if bucketNameQuery == "" {
		return nil, errormdl.Wrap("please provide value of bucketNameQuery")
	}

	b := Bucket{
		BucketID:        guidmdl.GetGUID(),
		BucketNameQuery: bucketNameQuery,
		IsDynamicName:   isDynamicName,
		BucketPath:      bucketPath,
	}
	bucket := AppendBucket{}
	bucket.Bucket = b
	return &bucket, nil
}

func (ab *AppendBucket) Insert(filePath string, data *gjson.Result) error {
	locker := locker.NewLocker(filePath)
	locker.Lock()
	appendFile, err := filetype.NewAppendFile(filePath, ab.Bucket.SecurityProvider)
	defer func() {
		appendFile.Close()
		locker.Unlock()
	}()
	if err != nil {
		return err
	}

	err = appendFile.Write(data)
	return err
}

func (ab *AppendBucket) Find(filePaths []string, queries []string, data *gjson.Result) (string, error) {
	return "", errormdl.Wrap("operation not allowed")
}

func (ab *AppendBucket) Update(filePaths []string, queries []string, data *gjson.Result) (*gjson.Result, []error) {
	return nil, []error{errormdl.Wrap("operation not allowed")}
}

func (ab *AppendBucket) Delete(filePaths, queries []string, data *gjson.Result) (recordsDeletedCnt int, errList []error) {
	return 0, []error{errormdl.Wrap("operation not allowed")}
}
