package bucket

import (
	"strings"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/corefdb/index"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/corefdb/securityprovider"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"github.com/tidwall/gjson"
)

const (
	PathSeperator     = "/"
	DynamicPathPrefix = "$$"
)

type PathProvider interface {
	GetPath(rs *gjson.Result) (string, error)
}

type Securable interface {
	Secure(securityprovider.SecurityProvider)
}
type Store interface {
	Insert(string, *gjson.Result) error
	Find([]string, []string, *gjson.Result) (string, error)
	Update([]string, []string, *gjson.Result) (*gjson.Result, []error)
	Delete([]string, []string, *gjson.Result) (int, []error)
}

type MediaStore interface {
	WriteMedia(filePath string, mediaData []byte, rs *gjson.Result) (string, error)
	ReadMedia(filePath string, recordID string) ([]byte, *gjson.Result, error)
	UpdateMedia(filePath string, recordID string, mediaData []byte, rs *gjson.Result) (err error)
	UpsertMedia(filePath string, recordID string, mediaData []byte, rs *gjson.Result) (string, error)
}

type Bucket struct {
	BucketID        string `json:"bucketId"`
	IsDynamicName   bool   `json:"isDynamicName"`
	BucketNameQuery string `json:"bucketNameQuery"`
	// TODO: rename to Indexex
	Indexes          []string `json:"indices"`
	BucketPath       string   `json:"bucketPath"`
	SecurityProvider securityprovider.SecurityProvider
}

func (bucket *Bucket) AddIndex(index *index.Index) error {
	if index == nil {
		return errormdl.Wrap("index value is nil")
	}
	bucket.Indexes = append(bucket.Indexes, index.IndexID)
	index.BucketSequence = append(index.BucketSequence, bucket.BucketID)
	return nil
}

// ResolveName - returns bucket name
func (bucket *Bucket) GetPath(rs *gjson.Result) (string, error) {
	path := ""
	pathChunks := strings.Split(bucket.BucketPath, PathSeperator)
	for i := range pathChunks {
		pathVal := pathChunks[i]
		if strings.HasPrefix(pathChunks[i], DynamicPathPrefix) {
			dynamicField := strings.TrimSpace(strings.TrimPrefix(pathChunks[i], DynamicPathPrefix))
			pathVal = strings.TrimSpace(rs.Get(dynamicField).String())
			if pathVal == "" {
				return "", errormdl.Wrap("please provide value for bucket name: " + dynamicField)
			}
		}
		path = path + PathSeperator + pathVal
	}
	name := bucket.BucketNameQuery
	if bucket.IsDynamicName {
		name = rs.Get(name).String()
	}
	if name == "" {
		return name, errormdl.Wrap("please provide value for bucket name: " + bucket.BucketNameQuery)
	}
	path = strings.TrimPrefix(path+PathSeperator+name, PathSeperator)
	return path, nil
}

func (bucket *Bucket) Secure(securityprovider securityprovider.SecurityProvider) {
	bucket.SecurityProvider = securityprovider
}
