package filetype

import (
	"os"
	"path/filepath"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/corefdb/securityprovider"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/filemdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	"github.com/tidwall/gjson"
)

type AppendFile struct {
	FilePath         string
	Fp               *os.File
	IsLazyEnable     bool
	securityProvider securityprovider.SecurityProvider
}

func NewAppendFile(filePath string, securityProvider securityprovider.SecurityProvider) (*AppendFile, error) {
	if filePath == "" {
		return nil, errormdl.Wrap("please provide valid filepath")
	}

	path, err := filepath.Abs(filePath)
	if err != nil {
		return nil, err
	}
	dir, _ := filepath.Split(path)
	if dir != "" {
		err := filemdl.CreateDirectoryRecursive(dir)
		if err != nil {
			return nil, err
		}
	}
	f, err := os.OpenFile(path, os.O_CREATE|os.O_RDWR, os.ModePerm)
	if err != nil {
		return nil, errormdl.Wrap("fail to open file: " + err.Error())
	}
	file := AppendFile{
		FilePath:         filePath,
		Fp:               f,
		securityProvider: securityProvider,
	}
	return &file, nil
}

func (af *AppendFile) Write(rs *gjson.Result) (err error) {
	dataBytes := []byte(rs.String())
	if af.securityProvider != nil {
		dataBytes, err = af.securityProvider.Encrypt(dataBytes, af.Fp.Name(), rs)
		if err != nil {
			loggermdl.LogError(err)
			return
		}
	}
	dataBytes = append(dataBytes, []byte(lineBreak)...)
	_, _, err = filemdl.AppendDataToFile(af.Fp.Name(), dataBytes, true)
	return err
}

func (af *AppendFile) Close() error {
	return af.Fp.Close()
}
