package lazycache

import (
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/cachemdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/lazywriter"
)

// Create Master lazy object

// IndexMaster - Holds master object for index files
var IndexMaster lazywriter.LazyFDBHelper

// AppendMaster - Holds master object for Append files
var AppendMaster lazywriter.LazyFDBHelper

// IndexLazyObjHolder - Holds lazy cache objects for indexes, indexID as key and lazy Object as value
var IndexLazyObjHolder cachemdl.FastCacheHelper

// AppendLazyObjHolder - Holds lazy cache objects for Append files, filepath as key and lazy Object as value
var AppendLazyObjHolder cachemdl.FastCacheHelper

// LazyWriteObjMaster - Holds master object for Append files
var LazyWriteObjMaster lazywriter.LazyFDBHelper

// LazyWriteObjHolder - Holds lazy cache objects for Append files, filepath as key and lazy Object as value
var LazyWriteObjHolder cachemdl.FastCacheHelper

const (
	maxObjetsCnt                  int = 100000
	maxRetryCnt                   int = 5
	intervalTimeForLazyIndexWrite int = 10
	intervalTimeForLazyWrite      int = 20
	sleepTime                     int = 5
	// NoExpiration -
	NoExpiration time.Duration = -1

	lazyIndexProcessName  string = "lazyIndex"
	lazyAppendProcessName string = "lazyAppend"
	lazyProcessName       string = "lazyWrite"
)

func init() {
	// loggermdl.LogError("init lazyIndex")
	// start process for lazy index file operations
	IndexMaster.StartProcess(maxObjetsCnt, lazyIndexProcessName, intervalTimeForLazyIndexWrite, sleepTime, maxRetryCnt, false)
	// loggermdl.LogError("Lazy writer process started")
	// initialize a map to hold lazy writer objects for the index file data
	IndexLazyObjHolder.Setup(10000, NoExpiration, NoExpiration)

	// start process for lazy Append file operations
	AppendMaster.StartProcess(maxObjetsCnt, lazyAppendProcessName, intervalTimeForLazyWrite, sleepTime, maxRetryCnt, false)

	// initialize a map to hold lazy writer objects for the append file data
	AppendLazyObjHolder.Setup(10000, NoExpiration, NoExpiration)
	// loggermdl.LogError("cache setup done")

	// start process for lazy Append file operations
	LazyWriteObjMaster.StartProcess(maxObjetsCnt, lazyProcessName, intervalTimeForLazyWrite, sleepTime, maxRetryCnt, false)

	// initialize a map to hold lazy writer objects for the append file data
	LazyWriteObjHolder.Setup(10000, NoExpiration, NoExpiration)
	// loggermdl.LogError("cache setup done")

}
