package coreimmudb

import (
	"reflect"
	"testing"
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	"github.com/codenotary/immudb/pkg/api/schema"
	immudb "github.com/codenotary/immudb/pkg/client"
)

func init() {
	// Creating and connecting two Immu database connection: ImmuDbHost1,ImmuDbHost4

	// err := InitImmuDBConnections([]ImmuHost{{HostName: "ImmuDbHost1", Server: "localhost", Port: 3322, Database: "defaultdb", Username: "immudb", Password: "immudb", Dir: "", MaxRecvMsgSize: 0, IsDefault: true, IsDisabled: false},
	// {HostName: "ImmuDbHost4", Server: "localhost", Port: 3322, Database: "defaultdb", Username: "immudb", Password: "immudb", Dir: "", MaxRecvMsgSize: 0, IsDefault: false, IsDisabled: false}})
	// if err != nil {
	// panic(err)
	// }

	// Establishing ImmuDB connection using dbconfig.json file
	// err := InitUsingJson("./dbconfig.json")
	// if err != nil {
	// 	panic(err)
	// }
}

func TestInitUsingJson(t *testing.T) {
	type args struct {
		filePath string
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{name: "Test 1", args: args{filePath: "./dbconfig.json"}, wantErr: false}, // PASS
		{name: "Test 2", args: args{filePath: "./sample.json"}, wantErr: true},    // FAIL - Config file does not exists
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := InitUsingJson(tt.args.filePath); (err != nil) != tt.wantErr {
				t.Errorf("InitUsingJsonFile() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func ExampleInitUsingJson() {
	filePath := "./dbconfig.json"
	// Establishing connection using JSON file
	err := InitUsingJson(filePath)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo("Immudb connection successfully established!")
}

// Test - InitImmuDBConnections
func TestInitImmuDBConnections(t *testing.T) {
	type args struct {
		hosts []ImmuHost
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{name: "Test 1", args: args{[]ImmuHost{{"LocalHostDB1", "localhost", 3322, "defaultdb", "immudb", "immudb", "", 0, true, false}}}, wantErr: false},       // PASS
		{name: "Test 2", args: args{[]ImmuHost{{"LocalHostDB2", "localhost", 3325, "defaultdb", "immudb", "immudb", "", 0, true, false}}}, wantErr: true},        // FAIL - Wrong connection details
		{name: "Test 3", args: args{[]ImmuHost{{"LocalHostDB3", "localhost", 3322, "wrongdb", "immudb", "immudb", "", 0, true, false}}}, wantErr: true},          // FAIL - Wrong connection details
		{name: "Test 4", args: args{[]ImmuHost{{"LocalHostDB4", "localhost", 3322, "defaultdb", "wrongUsername", "immudb", "", 0, true, false}}}, wantErr: true}, // FAIL - Wrong connection details
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := InitImmuDBConnections(tt.args.hosts); (err != nil) != tt.wantErr {
				t.Errorf("InitImmuDBConnections() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func ExampleInitImmuDBConnections() {
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	host2 := ImmuHost{HostName: "Host2", Server: "127.0.0.1", Port: 3322, Database: "database1", Username: "username", Password: "password", Dir: "./immudb"}
	// Establishing database connection
	err := InitImmuDBConnections([]ImmuHost{host1, host2})
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo("Immudb connection established!")
}

// Test - AddDBConnection
func TestAddDBConnection(t *testing.T) {
	type args struct {
		host ImmuHost
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{name: "Test 1", args: args{host: ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password"}}, wantErr: false},     // PASS
		{name: "Test 2", args: args{host: ImmuHost{HostName: "Host2", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "WrongPassword"}}, wantErr: true}, // FAIL - Wrong credentails
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := AddDBConnection(tt.args.host); (err != nil) != tt.wantErr {
				t.Errorf("AddDBConnection() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func ExampleAddDBConnection() {
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	// Establishing database connection for above host details
	err := AddDBConnection(host1)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo("Immudb single connection successfully established!")
}

// Test to disconnect ImmuDb connection / session
func TestDeleteImmuDbSession(t *testing.T) {
	type args struct {
		hostName string
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{name: "Test 1", args: args{hostName: "ImmuDbHost1"}, wantErr: false}, // PASS
		{name: "Test 2", args: args{hostName: "ImmuDbHost2"}, wantErr: true},  // FAIL - Connection not found
		{name: "Test 3", args: args{hostName: "ImmuDbHost3"}, wantErr: true},  // FAIL - Connection not found
		{name: "Test 4", args: args{hostName: "ImmuDbHost4"}, wantErr: false}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := DeleteImmuDbSession(tt.args.hostName); (err != nil) != tt.wantErr {
				t.Errorf("DeleteImmuDbSession() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func ExampleDeleteImmuDbSession() {
	// Connecting to the Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	// Establishing database connection
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Closing database connection (session)
	err = DeleteImmuDbSession(host1.HostName)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(host1.HostName + " connection closed!")
}

func TestImmuDbConnection(t *testing.T) {
	type args struct {
		host ImmuHost
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{name: "Test 1", args: args{host: ImmuHost{"ImmuDbHost1", "localhost", 3322, "defaultdb", "immudb", "immudb", "", 0, true, false}}, wantErr: true},   // PASS
		{name: "Test 2", args: args{host: ImmuHost{"ImmuDbHost2", "localhost", 3324, "defaultdb", "immudb", "immudb", "", 0, false, false}}, wantErr: false}, // FAIL - Immudb not running at provided port
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if err := TestDbConnection(tt.args.host); (err != nil) != tt.wantErr {
				t.Errorf("TestDbConnection() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func ExampleTestDbConnection() {
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := TestDbConnection(host1)
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo("Test connection successful!")
}

func TestGetImmuDbConnection(t *testing.T) {
	type args struct {
		hostName string
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		args    args
		want    immudb.ImmuClient
		wantErr bool
	}{
		{name: "Test 1", args: args{hostName: "ImmuDbHost1"}, wantErr: false}, // PASS
		{name: "Test 2", args: args{hostName: "ImmuDbHost2"}, wantErr: true},  // FAIL - Connection not found
		{name: "Test 3", args: args{hostName: "ImmuDbHost3"}, wantErr: true},  // FAIL - Connection not found
		{name: "Test 4", args: args{hostName: "ImmuDbHost4"}, wantErr: false}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := GetImmuDbConnection(tt.args.hostName)
			if (err != nil) != tt.wantErr {
				t.Errorf("getImmuDbConnection() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("getImmuDbConnection() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleGetImmuDbConnection() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting immudb.ImmuClient
	client, err := GetImmuDbConnection(host1.HostName)
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	_ = client
}

func TestGetImmuDAOWithHost(t *testing.T) {
	type args struct {
		hostname string
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name string
		args args
		want *ImmuDAO
	}{
		{name: "Test 1", args: args{hostname: "ImmuDbHost1"}},
		{name: "Test 2", args: args{hostname: "ImmuDbHost2"}},
		{name: "Test 3", args: args{hostname: "ImmuDbHost3"}},
		{name: "Test 4", args: args{hostname: "ImmuDbHost4"}}}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got := GetImmuDAOWithHost(tt.args.hostname)
			if reflect.DeepEqual(got, nil) {
				t.Errorf("GetImmuDAOWithHost() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleGetImmuDAOWithHost() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// For getting specific connection immudb Data Access Object(DAO), just provide connection hostname.
	immuDAO := GetImmuDAOWithHost(host1.HostName)

	// For getting default connection Data Access Object(DAO)
	immuDAO = GetImmuDAOWithHost(DefaultHostName)

	_ = immuDAO
}

func TestImmuDAO_GetKeyData(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		key []byte
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.Entry
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Akshay")}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost2"}, args: args{key: []byte("Demo")}, wantErr: true},    // FAIL - Connection not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Demo")}, wantErr: true},    // FAIL - Key not found
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost4"}, args: args{key: []byte("key")}, wantErr: false},    // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.GetKeyData(tt.args.key)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.GetKeyData() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.GetKeyData() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleGetKeyData() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)

	// You can set GetOptions for advance filtering as show below;
	// immuDAO.MetaData.GetOpts = GetOptions{AtTx: 0, AtRevision: 0, SinceTx: 0, NoWait: true}

	key := "key1"
	result, err := immuDAO.GetKeyData([]byte(key))
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(result)

	// Output
	// [tx:1 key:"key1" value:"key1value" revision:1]
}

func TestImmuDAO_GetVerifiedKeyData(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		key []byte
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.Entry
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Akshay")}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Demo")}, wantErr: true},    // FAIL - Wrong key
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost2"}, args: args{key: []byte("Demo")}, wantErr: true},    // FAIL - Connection not found
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost4"}, args: args{key: []byte("key")}, wantErr: false},    // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.GetVerifiedKeyData(tt.args.key)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.GetVerifiedKeyData() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.GetVerifiedKeyData() = %v, want %v", got, tt.want)
			}
		})
	}
}
func ExampleGetVerifiedKeyData() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)

	// You can set GetOptions for advance filtering as show below;
	// immuDAO.MetaData.GetOpts = GetOptions{AtTx: 0, AtRevision: 0, SinceTx: 0, NoWait: true}

	key := "key1"
	result, err := immuDAO.GetVerifiedKeyData([]byte(key))
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(result)

	// Output:
	// [tx:1 key:"key1" value:"key1value" revision:1]
}

func TestImmuDAO_GetAllKeysData(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		keys [][]byte
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.Entries
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{keys: [][]byte{[]byte("Akshay"), []byte("key")}}, wantErr: false},    // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost1"}, args: args{keys: [][]byte{[]byte("nokey1"), []byte("nokey2")}}, wantErr: false}, // PASS - but empty result due no keys found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost2"}, args: args{keys: [][]byte{[]byte("nokey1"), []byte("nokey2")}}, wantErr: true},  // FAIL - Connection not found
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.GetAllKeysData(tt.args.keys)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.GetAllKeysData() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.GetAllKeysData() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleGetAllKeysData() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	key1 := "key1"
	key2 := "key2"

	keys := [][]byte{[]byte(key1), []byte(key2)}
	result, err := immuDAO.GetAllKeysData(keys)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(result)
	// Output:
	// [entries:{tx:1 key:"key1" value:"key1value" revision:1} entries:{tx:2 key:"key2" value:"key2value" revision:1}]
}
func TestImmuDAO_GetKeyHistory(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		key    []byte
		offset uint64
		limit  int32
		desc   bool
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.Entries
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Akshay"), offset: 0, limit: 0, desc: false}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost2"}, args: args{key: []byte("nokey"), offset: 0, limit: 0, desc: false}, wantErr: true},   // FAIL - Connection not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("nokey"), offset: 0, limit: 0, desc: false}, wantErr: true},   // FAIL - Key not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost4"}, args: args{key: []byte("key"), offset: 0, limit: 0, desc: false}, wantErr: false},    // PASS

	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.GetKeyHistory(tt.args.key, tt.args.offset, tt.args.limit, tt.args.desc)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.GetKeyHistory() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.GetKeyHistory() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleGetKeyHistory() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	key := "key2"

	offset := 0  // skipping the key entries
	limit := 0   // limiting number of entries
	desc := true // sorting in descending order
	result, err := immuDAO.GetKeyHistory([]byte(key), uint64(offset), int32(limit), desc)
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)
	// Output:
	// [entries:{tx:2 key:"key2" value:"key2value-2" revision:2} entries:{tx:1 key:"key2" value:"key2value" revision:1}]
}
func TestImmuDAO_SetKeyData(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		key   []byte
		value []byte
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.TxHeader
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Akshay"), value: []byte("Mahendrakar")}, wantErr: false},                                                       // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost2"}, args: args{key: []byte("immu"), value: []byte("db")}, wantErr: true},                                                                   // FAIL - Connection not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("valueEmpty"), value: []byte("")}, wantErr: false},                                                              // FAIL
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte(""), value: []byte("keyEmpty")}, wantErr: true},                                                                 // FAIL - Key is empty
		{name: "Test 5", fields: fields{HostName: "ImmuDbHost4"}, args: args{key: []byte("jsonStringObj"), value: []byte("{'first_name':'Akshay', 'last_name':'Mahendrakar', 'company':'MKCL Pune' }")}}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.SetKeyData(tt.args.key, tt.args.value)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.SetKeyData() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.SetKeyData() = %v, want %v", got, tt.want)
			}
		})
	}
}
func ExampleSetKeyData() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	key := "key1"
	value := "key1value"

	result, err := immuDAO.SetKeyData([]byte(key), []byte(value))
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)
	// Output:
	// [id:35645 prevAlh:"\xfb\x92d\xb4:$D2\x17O\"\x85\xb5\x9e\xe6\xf4]\x97\xff+\x8e8\xfe\x00/\xceQD\x89W\xb4o" ts:1688540404 nentries:1 eH:"\xbd9\x90 [t>\xda\xeeA_V\r^'\xefm\xec/ $>\xa5\xe7\x17\xf1\xb9\xb8C{\x9d\x8c" blTxId:35644 blRoot:"7\x98w\xea\x13\x07\xf4\x16>\"f.\x8a\xabk\xe2s\xc0\xe4\xee1j\x884\t\x81\xa6\xb9\x82\x14f&" version:1 metadata:{}]
}

func TestImmuDAO_SetVerifiedKeyData(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		key   []byte
		value []byte
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.TxHeader
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("Akshay"), value: []byte("Mahendrakar")}, wantErr: false},                                                       // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost2"}, args: args{key: []byte("immu"), value: []byte("db")}, wantErr: true},                                                                   // FAIL - Connection not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("valueEmpty"), value: []byte("")}, wantErr: false},                                                              // FAIL
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte(""), value: []byte("keyEmpty")}, wantErr: true},                                                                 // FAIL - Key is empty
		{name: "Test 5", fields: fields{HostName: "ImmuDbHost4"}, args: args{key: []byte("jsonStringObj"), value: []byte("{'first_name':'Akshay', 'last_name':'Mahendrakar', 'company':'MKCL Pune' }")}}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.SetVerifiedKeyData(tt.args.key, tt.args.value)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.SetVerifiedKeyData() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.SetVerifiedKeyData() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleSetVerifiedKeyData() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	key := "key1"
	value := "key1value"

	result, err := immuDAO.SetVerifiedKeyData([]byte(key), []byte(value))
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)
	// Output:
	// [id:35648 prevAlh:"\x1e\x93\xfc˩\xbcn\x8d,Ѭ\x7f\x8ac\xb1f\xd6\xfb\x1f\xe8\x8d'J\x85\xcf\x0f\xf9\xba\xf30\x84\xea" ts:1688540894 nentries:1 eH:"0\xe8+V\x15f5m\x8b\xfae\xc4,\x11\x15G[\x9c%\r\x07\xc1\xdd\xd8\xfcu{\x9a\xc8h\xc3N" blTxId:35647 blRoot:"dzAČ\x19\xc1\x8cX\xc1\xb0\xa7w\xcbm\xc3K\xbck\xa7\x9a\xaf\rl\x1b\xa75\xb8\xf9ѠM" version:1 metadata:{}]
}

func TestImmuDAO_SetAllKeysData(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		kv []KeyValue
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.TxHeader
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{kv: []KeyValue{{Key: []byte("test1"), Value: []byte("test1 value")}, {Key: []byte("test2"), Value: []byte("test2 value")}}}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost2"}, args: args{kv: []KeyValue{{Key: []byte("test3"), Value: []byte("test3 value")}, {Key: []byte("test4"), Value: []byte("test4 value")}}}, wantErr: true},  // FAIL - Connection not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost4"}, args: args{kv: []KeyValue{{Key: []byte(""), Value: []byte("test3 value")}, {Key: []byte("test4"), Value: []byte("test4 value")}}}, wantErr: true},       // FAIL - Key not set
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost4"}, args: args{kv: []KeyValue{{Key: []byte("test3"), Value: []byte("test3 value")}, {Key: []byte("test4"), Value: []byte("test4 value")}}}, wantErr: false}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.SetAllKeysData(tt.args.kv)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.SetAllKeysData() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.SetAllKeysData() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleSetAllKeysData() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)

	kv1 := KeyValue{Key: []byte("key1"), Value: []byte("key1value")}
	kv2 := KeyValue{Key: []byte("key2"), Value: []byte("key2value")}

	kvs := []KeyValue{kv1, kv2}

	result, err := immuDAO.SetAllKeysData(kvs)
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)
	// Output:
	// [id:35649  prevAlh:"\x1c\xa5\xa4\xd7\xefB\x9bpǛ\xa3{\x1e\xf5(\x1b\xc8)\xc6Q\xe4ŏo\x87\xe6]\x12\xc2BӪ"  ts:1688541071  nentries:2  eH:"D\xdbVv\x9a*\xce\xd2r P\\\xca\xca\x12\xd1͔\xa7\x86\xda%\x96\xf1\xaf~\xa6\xb2\xfb\xf9]e"  blTxId:35648  blRoot:"\xc32\xf4d\xd8i\xa9\x0b\xe6\xd2\xe8\xc2\x11\ttș\xb6\x1e\n\xd5u\xf66\xdf\xc4N;\xcfϷ\x9a"  version:1  metadata:{}]
}

func TestImmuDAO_DeleteKey(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		keys [][]byte
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.TxHeader
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{keys: [][]byte{[]byte("test 3"), []byte("test4")}}, wantErr: true}, // FAIL - Key not found "test 3"
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost2"}, args: args{keys: [][]byte{[]byte("test 3"), []byte("test4")}}, wantErr: true}, // FAIL - Connection not found
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost1"}, args: args{keys: [][]byte{[]byte("test3"), []byte("test4")}}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost4"}, args: args{keys: [][]byte{[]byte("test3"), []byte("test4")}}, wantErr: true},  // FAIL - Key not found

	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.DeleteKey(tt.args.keys)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.DeleteKey() error = %v, wantErr %v", err, tt.wantErr)
				return
			}

			if reflect.DeepEqual("got", tt.want) && err == nil {
				t.Errorf("ImmuDAO.DeleteKey() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleDeleteKey() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	keys := [][]byte{[]byte("key1")}
	result, err := immuDAO.DeleteKey(keys)
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)
	// Output:
	// [id:35651 prevAlh:"u\x1bH\xd6s\x11\x93\xf5\xd8xRj\xb9\xcbXd\xdeO\xa8\xa3\x0em\x8bj\xaaJ\xb1\x1bO|\x0f\xf6" ts:1688541183 nentries:1 eH:"4\x14\x18\x02ˆSnV\xa3\x9a\x84฀||\x92'\xea\xcc\xee\x12w\xba\x88\xe0'נf\x94" blTxId:35650 blRoot:"\x94+\xc3^\xb2\xc4\xc3&\xa3Q2\xc2\xe6\x87\x07\x93\xd3\xe5U\x96\x14\xc7?7/\"\x9cV\xd8jep" version:1 metadata:{}]
}

func TestImmuDAO_ExpirableKeySet(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		key       []byte
		value     []byte
		expiresAt time.Time
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.TxHeader
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{key: []byte("test5"), value: []byte("test5 value"), expiresAt: time.Now().Add(2 * time.Minute)}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost2"}, args: args{key: []byte("test6"), value: []byte("test6 value"), expiresAt: time.Now().Add(2 * time.Minute)}, wantErr: true},  // FAIL - Connection not found
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost"}, args: args{key: []byte(""), value: []byte("test6 value"), expiresAt: time.Now().Add(2 * time.Minute)}, wantErr: true},        // FAIL - Key not found
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost4"}, args: args{key: []byte("test6"), value: []byte("test6 value"), expiresAt: time.Now().Add(2 * time.Minute)}, wantErr: false}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.ExpirableKeySet(tt.args.key, tt.args.value, tt.args.expiresAt)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.ExpirableKeySet() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.ExpirableKeySet() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleExpirableKeySet() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	key := "key1"
	value := "key1value"
	expireTime := time.Now().Add(time.Minute * 5) // Setting key expiration time to 5 minutes
	result, err := immuDAO.ExpirableKeySet([]byte(key), []byte(value), expireTime)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(result)

	/*Output:
	 [id:35652 prevAlh:"\xe4l\"\x05\x83\xb0?\x9c\x9cl\xf1\x7f\xf3+\x1d\xf8\xfa\n3\x0b֘\x14\xad\x94bX\x07\xe5\xcdB\xd7" ts:1688541334 nentries:1 eH:"\x87\xed\xe4x\x02\x8a\xe6\x14y\xdf\x1e\x8b\xfe\x0cɑ\xe0\x0b\xbf\xf2stv\xdf\xde\x03 \x91w\x9e4@" blTxId:35651 blRoot:"\xf4\xa5\xb7\xed\xd2UT\xfa\xb4\x92\x88S5\x99\x7f\xf1΍3H\x8a\x99I\xba\x95\xe6\xbe\x1c\xacg\xa2S" version:1 metadata:{}]

	// metadata field contains expiration time (GetKeyData)
	 [tx:35652 key:"key1" value:"key1value" metadata:{expiration:{expiresAt:1688541634}} revision:4]*/
}

func TestImmuDAO_CreateDatabase(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		dbName     string
		dbSettings *schema.DatabaseNullableSettings
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.CreateDatabaseResponse
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{dbName: "db1", dbSettings: nil}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost3"}, args: args{dbName: "db2", dbSettings: nil}, wantErr: true},  // FAIL - Connection not found for host
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost4"}, args: args{dbName: "", dbSettings: nil}, wantErr: true},     // FAIL - Database name cannot be empty

	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.CreateDatabase(tt.args.dbName, tt.args.dbSettings)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.CreateDatabase() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.CreateDatabase() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleCreateDatabase() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	dbName := "Database1"
	result, err := immuDAO.CreateDatabase(dbName, nil)
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)
	// Output:
	// [name:"Database1"  settings:{replicationSettings:{replica:{}  primaryDatabase:{}  primaryHost:{}  primaryPort:{}  primaryUsername:{}  primaryPassword:{}  syncReplication:{}  syncAcks:{}  prefetchTxBufferSize:{}  replicationCommitConcurrency:{}  allowTxDiscarding:{}  skipIntegrityCheck:{}  waitForIndexing:{}}  fileSize:{value:536870912}  maxKeyLen:{value:1024}  maxValueLen:{value:33554432}  maxTxEntries:{value:1024}  excludeCommitTime:{}  maxConcurrency:{value:30}  maxIOConcurrency:{value:1}  txLogCacheSize:{value:1000}  vLogMaxOpenedFiles:{value:10}  txLogMaxOpenedFiles:{value:10}  commitLogMaxOpenedFiles:{value:10}  indexSettings:{flushThreshold:{value:100000}  syncThreshold:{value:1000000}  cacheSize:{value:100000}  maxNodeSize:{value:4096}  maxActiveSnapshots:{value:100}  renewSnapRootAfter:{value:1000}  compactionThld:{value:2}  delayDuringCompaction:{value:10}  nodesLogMaxOpenedFiles:{value:10}  historyLogMaxOpenedFiles:{value:1}  commitLogMaxOpenedFiles:{value:1}  flushBufferSize:{value:4096}  cleanupPercentage:{}  maxBulkSize:{value:1}  bulkPreparationTimeout:{value:20}}  writeTxHeaderVersion:{value:1}  autoload:{value:true}  readTxPoolSize:{value:128}  syncFrequency:{value:20}  writeBufferSize:{value:4194304}  ahtSettings:{syncThreshold:{value:100000}  writeBufferSize:{value:16777216}}  maxActiveTransactions:{value:1000}  mvccReadSetLimit:{value:100000}  vLogCacheSize:{}  truncationSettings:{retentionPeriod:{}  truncationFrequency:{value:86400000}}  embeddedValues:{}  preallocFiles:{}}]
}

func TestImmuDAO_DeleteDatabase(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		dbName string
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *schema.DeleteDatabaseResponse
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, args: args{dbName: "db1"}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost3"}, args: args{dbName: "db2"}, wantErr: true},  // FAIL - Connection not found for host
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost4"}, args: args{dbName: "db3"}, wantErr: true},  // FAIL - Database not found
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			got, err := i.DeleteDatabase(tt.args.dbName)
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.DeleteDatabase() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if reflect.DeepEqual(got, tt.want) && err == nil {
				t.Errorf("ImmuDAO.DeleteDatabase() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleDeleteDatabase() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	dbName := "defaultdb"
	result, err := immuDAO.DeleteDatabase(dbName)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(result)
	// Output:
	// [database:"defaultdb"]
}

func TestImmuDAO_CreateUser(t *testing.T) {
	type fields struct {
		HostName string
	}
	type args struct {
		username   string
		password   string
		dbName     string
		permission int
	}
	// NOTE: ImmuDbHost1 and ImmuDbHost4 database connection are created at init method
	tests := []struct {
		name    string
		fields  fields
		args    args
		wantErr bool
	}{
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost4"}, args: args{username: "newuser10", password: "Password@2", dbName: "defaultdb", permission: ReadPermission}, wantErr: false},      // PASS - Read Permission
		{name: "Test 3", fields: fields{HostName: "ImmuDbHost1"}, args: args{username: "newuser20", password: "Password@3", dbName: "defaultdb", permission: ReadWritePermission}, wantErr: false}, // PASS - Read Write Permission
		{name: "Test 4", fields: fields{HostName: "ImmuDbHost4"}, args: args{username: "newuser30", password: "Password@4", dbName: "defaultdb", permission: AdminPermission}, wantErr: false},     // PASS - Admin Permission
		{name: "Test 5", fields: fields{HostName: "ImmuDbHost4"}, args: args{username: "newuser40", password: "Password", dbName: "defaultdb", permission: AdminPermission}, wantErr: true},        // FAIL - Password criteria does not match
		{name: "Test 5", fields: fields{HostName: "ImmuDbHost1"}, args: args{username: "newuser50", password: "Password@123", dbName: "defaultdb", permission: 200}, wantErr: true},                // FAIL - Invalid user permissions
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
			}
			if err := i.CreateUser(tt.args.username, tt.args.password, tt.args.dbName, tt.args.permission); (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.CreateUser() error = %v, wantErr %v", err, tt.wantErr)
			}
		})
	}
}

func ExampleCreateUser() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	username := "user1"
	password := "user1P@ssw0rd"
	dbName := "defaultdb"

	// AdminPermission
	// ReadPermission
	// ReadWritePermission
	permission := AdminPermission

	err = immuDAO.CreateUser(username, password, dbName, permission)
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(username + " user created successfully!")
}
func TestImmuDAO_ListUsers(t *testing.T) {
	type fields struct {
		HostName string
		MetaData MetaData
	}
	tests := []struct {
		name    string
		fields  fields
		want    *schema.UserList
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost10"}, wantErr: true}, // FAIL
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
				MetaData: tt.fields.MetaData,
			}
			got, err := i.ListUsers()
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.ListUsers() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) && err != nil {
				t.Errorf("ImmuDAO.ListUsers() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleListUsers() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	result, err := immuDAO.ListUsers()
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	loggermdl.LogInfo(result)
	// Output:
	// [user:"user1" permissions:{database:"defaultdb" permission:254} createdby:"immudb" createdat:"2023-07-05 14:06:39.187034384 +0530 IST" active:true]
}

func TestImmuDAO_ListDatabases(t *testing.T) {
	type fields struct {
		HostName string
		MetaData MetaData
	}
	tests := []struct {
		name    string
		fields  fields
		want    *schema.DatabaseListResponseV2
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost10"}, wantErr: true}, // FAIL
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
				MetaData: tt.fields.MetaData,
			}
			got, err := i.ListDatabases()
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.ListDatabases() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) && err != nil {
				t.Errorf("ImmuDAO.ListDatabases() = %v, want %v", got, tt.want)
			}
		})
	}
}
func ExampleListDatabases() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	result, err := immuDAO.ListDatabases()
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	loggermdl.LogInfo(result)

	// Output:
	// [name:"testdb" settings:{replicationSettings:{replica:{} primaryDatabase:{} primaryHost:{} primaryPort:{} primaryUsername:{} primaryPassword:{} syncReplication:{} syncAcks:{} prefetchTxBufferSize:{} replicationCommitConcurrency:{} allowTxDiscarding:{} skipIntegrityCheck:{} waitForIndexing:{}} fileSize:{value:536870912} maxKeyLen:{value:1024} maxValueLen:{value:33554432} maxTxEntries:{value:1024} excludeCommitTime:{} maxConcurrency:{value:30} maxIOConcurrency:{value:1} txLogCacheSize:{value:1000} vLogMaxOpenedFiles:{value:10} txLogMaxOpenedFiles:{value:10} commitLogMaxOpenedFiles:{value:10} indexSettings:{flushThreshold:{value:100000} syncThreshold:{value:1000000} cacheSize:{value:100000} maxNodeSize:{value:4096} maxActiveSnapshots:{value:100} renewSnapRootAfter:{value:1000} compactionThld:{value:2} delayDuringCompaction:{value:10} nodesLogMaxOpenedFiles:{value:10} historyLogMaxOpenedFiles:{value:1} commitLogMaxOpenedFiles:{value:1} flushBufferSize:{value:4096} cleanupPercentage:{} maxBulkSize:{value:1} bulkPreparationTimeout:{value:20}} writeTxHeaderVersion:{value:1} autoload:{value:true} readTxPoolSize:{value:128} syncFrequency:{value:20} writeBufferSize:{value:4194304} ahtSettings:{syncThreshold:{value:100000} writeBufferSize:{value:16777216}} maxActiveTransactions:{value:1000} mvccReadSetLimit:{value:100000} vLogCacheSize:{} truncationSettings:{retentionPeriod:{value:86400000} truncationFrequency:{value:86400000}} embeddedValues:{} preallocFiles:{}} loaded:true]
}

func TestImmuDAO_GetActiveKeys(t *testing.T) {
	type fields struct {
		HostName string
		MetaData MetaData
	}
	tests := []struct {
		name    string
		fields  fields
		want    *schema.Entries
		wantErr bool
	}{
		{name: "Test 1", fields: fields{HostName: "ImmuDbHost1"}, wantErr: false}, // PASS
		{name: "Test 2", fields: fields{HostName: "ImmuDbHost4"}, wantErr: false}, // PASS
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			i := &ImmuDAO{
				HostName: tt.fields.HostName,
				MetaData: tt.fields.MetaData,
			}
			got, err := i.GetActiveKeys()
			if (err != nil) != tt.wantErr {
				t.Errorf("ImmuDAO.GetActiveKeys() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) && err != nil {
				t.Errorf("ImmuDAO.GetActiveKeys() = %v, want %v", got, tt.want)
			}
		})
	}
}

func ExampleGetActiveKeys() {
	// Establishing connection to Immudb
	host1 := ImmuHost{HostName: "Host1", Server: "127.0.0.1", Port: 3322, Database: "defaultdb", Username: "username", Password: "password", Dir: "./immudb", IsDefault: true}
	err := InitImmuDBConnections([]ImmuHost{host1})
	if err != nil {
		loggermdl.LogError(err)
		return
	}

	// Getting DAO (Data access object)
	immuDAO := GetImmuDAOWithHost(host1.HostName)
	// You can set ScanOptions for advance filtering as show below;
	// immuDAO.MetaData.ScanOpts = schema.ScanRequest{Desc: true, Limit: 10}

	result, err := immuDAO.GetActiveKeys()
	if err != nil {
		loggermdl.LogError(err)
		return
	}
	_ = result
	// Output:
	// result -> returns all the keys with value from the database
	// NOTE: It does not retrieve deleted keys!
}
