//@author  Ajit Jagtap
//@version Wed Jul 04 2018 20:15:13 GMT+0530 (IST)

// Package dalmdl will help you access
package dalmdl

import (
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/fdb"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	"github.com/tidwall/gjson"
)

// BaseDAO help to fetch data
type BaseDAO struct {
	filePath      string
	query         []string
	Output        gjson.Result
	isCacheable   bool
	cacheDuration time.Duration
}

// DAOBuilder will help to run gson query
type DAOBuilder struct {
	BaseDAO
}

// GetDAO return new instance
func GetDAO() *DAOBuilder {
	return &DAOBuilder{}
}

// Run : Execute Query
func (db *DAOBuilder) Run() (*gjson.Result, error) {
	db.Output = gjson.Result{}
	var err error
	for _, qry := range db.query {
		db.Output, err = fdb.GetDataDAO(db.filePath, qry, db.isCacheable, db.cacheDuration, db.Output)
		if errormdl.CheckErr(err) != nil {
			loggermdl.LogError("Error in executing query: ", qry, err)
			return nil, errormdl.CheckErr(err)
		}
	}

	return &db.Output, nil
}

// FilePath to open File and get data from FDB
func (db *DAOBuilder) FilePath(name string) *DAOBuilder {
	db.BaseDAO.filePath = name
	return db
}

// IsCacheableWithDuration to open File and get data from FDB
func (db *DAOBuilder) IsCacheableWithDuration(duration time.Duration) *DAOBuilder {
	if duration != 0 {
		db.BaseDAO.isCacheable = true
		db.BaseDAO.cacheDuration = duration
	}
	return db
}

// IsCacheable to open File and get data from FDB
func (db *DAOBuilder) IsCacheable() *DAOBuilder {
	db.BaseDAO.isCacheable = true
	return db
}

// Query give multiple queries for and condition
func (db *DAOBuilder) Query(query ...string) *DAOBuilder {
	db.BaseDAO.query = query
	return db
}
