package mysql

import (
	"fmt"
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	"github.com/gocraft/dbr/v2"
	"github.com/gocraft/dbr/v2/dialect"
	"github.com/stretchr/testify/assert"
)

func TestInit(t *testing.T) {
	connmap, err := Init("mysql-config.toml", "localhost")
	if err != nil {
		loggermdl.LogError("err: ", err)
	}
	db := connmap["localhost"]
	stmt, _ := db.Prepare("INSERT person SET Name=?")
	stmt.Exec("astaxie1")
	assert.NoError(t, err, "This should not return error")
}
func TestInitByPing(t *testing.T) {
	connmap, err := Init("mysql-config.toml", "localhost")
	if err != nil {
		loggermdl.LogError("err: ", err)
	}
	db := connmap["localhost"]
	err = db.Ping()
	if err != nil {
		panic(err.Error()) // proper error handling instead of panic in your app
	}
	assert.NoError(t, err, "This should not return error")
}

func TestInitWrongDefaultHostname(t *testing.T) {
	connmap, err := Init("mysql-config.toml", "localhostWrong")
	if err != nil {
		loggermdl.LogError("err: ", err)
	}
	db := connmap["localhost"]
	stmt, _ := db.Prepare("INSERT person SET Name=?")
	stmt.Exec("astaxie1")
	assert.NoError(t, err, "This should not return error")
}

func TestInitMultipleConnections(t *testing.T) {
	connmap, err := Init("mysql-config.toml", "localhost")
	if err != nil {
		loggermdl.LogError("err: ", err)
	}
	db := connmap["dev"]
	err2 := db.Ping()
	if err2 != nil {
		panic(err2.Error()) // proper error handling instead of panic in your app
	}
	stmt, _ := db.Prepare("INSERT person SET Name=?")
	stmt.Exec("astaxie111")
	assert.NoError(t, err, "This should not return error")
}

// func TestInitErrorWrongTomlFilePath(t *testing.T) {
// 	_, err := Init("mysql-config1.toml", "")
// 	assert.Error(t, err, "This should return error")
// }

// func TestInitError1(t *testing.T) {
// 	errormdl.IsTestingNegetiveCaseOn1 = true
// 	_, err := Init("mysql-config.toml", "localhost")
// 	errormdl.IsTestingNegetiveCaseOn1 = false
// 	assert.Error(t, err, "This should return error")
// }
func TestGetMYSQLConnection(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	db, err := GetMYSQLConnection("localhost")
	stmt, _ := db.Prepare("INSERT person SET Name=?")
	stmt.Exec("rr1")
	assert.NoError(t, err, "This should not return error")
}

func TestGetMYSQLConnectionEmptyHost(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	_, err := GetMYSQLConnection("")
	assert.NoError(t, err, "This should not  return error")
}
func TestGetMYSQLConnectionWrongHost(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	_, err := GetMYSQLConnection("localhost2")
	assert.Error(t, err, "This should return error")
}
func TestGetMYSQLConnectionWrongDefaultHost(t *testing.T) {
	Init("mysql-config.toml", "localhost2")
	_, err := GetMYSQLConnection("localhost")
	assert.NoError(t, err, "This should not return error")
}

// func TestGetMYSQLConnectionWrongDefaultHostAndEmptyHost(t *testing.T) {
// 	Init("mysql-config.toml", "localhost2")
// 	_, err := GetMYSQLConnection("")
// 	assert.Error(t, err, "This should return error")
// }

func TestGetMYSQLConnectionError(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOnCheckBool = true
	_, err := GetMYSQLConnection("localhost123")
	errormdl.IsTestingNegetiveCaseOnCheckBool = false
	assert.Error(t, err, "This should return error")
}
func TestSaveUpdateOrDeleteData(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	err := GetMysqlDAO().SaveUpdateOrDeleteData(insertstmt1, "mayurishinde15", 25, true, 2000.00)
	assert.NoError(t, err, "This should not return error")
}

func TestSaveUpdateOrDeleteDataInitError(t *testing.T) {
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	err := GetMysqlDAO().SaveUpdateOrDeleteData(insertstmt1, "mayuri2", 25, true, 2000.00)
	assert.Error(t, err, "This should return error")
}
func TestSaveUpdateOrDeleteDataError(t *testing.T) {
	Init("mysql-config.toml", "localhost1")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	errormdl.IsTestingNegetiveCaseOn = true
	err := GetMysqlDAOWithHost("localhost").SaveUpdateOrDeleteData(insertstmt1, "mayuri2", 25, true, 2000.00)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should return error")
}
func TestSaveUpdateOrDeleteDataError1(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	errormdl.IsTestingNegetiveCaseOn1 = true
	err := GetMysqlDAO().SaveUpdateOrDeleteData(insertstmt1, "mayuri2", 25, true, 2000.00)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should return error")
}
func TestSaveUpdateOrDeleteDataMysqlError(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	err := GetMysqlDAO().SaveUpdateOrDeleteData(insertstmt1, "mayuri2", 25, true, 2000.00)
	assert.Error(t, err, "This should return error")
}
func TestSaveUpdateOrDeleteDataError2(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	errormdl.IsTestingNegetiveCaseOn2 = true
	err := GetMysqlDAO().SaveUpdateOrDeleteData(insertstmt1, "mayuri2", 25, true, 2000.00)
	errormdl.IsTestingNegetiveCaseOn2 = false
	assert.Error(t, err, "This should return error")
}

func TestSaveUpdateOrDeleteDataError3(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	errormdl.IsTestingNegetiveCaseOn3 = true
	err := GetMysqlDAO().SaveUpdateOrDeleteData(insertstmt1, "mayuri3", 25, true, 3000.00)
	errormdl.IsTestingNegetiveCaseOn3 = false
	assert.Error(t, err, "This should return error")
}

func TestGetData(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	selectstmt := "SELECT count(*) cnt,age FROM person1 WHERE age =?"
	_, err := GetMysqlDAO().GetData(selectstmt, 25)
	assert.NoError(t, err, "This should not return error")
}
func TestGetDataError(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	selectstmt := "SELECT * FROM person1 WHERE age =?"
	errormdl.IsTestingNegetiveCaseOn = true
	_, err := GetMysqlDAO().GetData(selectstmt, 25)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should not return error")
}

func TestGetDataError1(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	selectstmt := "SELECT * FROM person1 WHERE age =?"
	errormdl.IsTestingNegetiveCaseOn1 = true
	_, err := GetMysqlDAO().GetData(selectstmt, 25)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should not return error")
}
func TestGetDataError2(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	selectstmt := "SELECT name,age FROM person1 WHERE age =?"
	errormdl.IsTestingNegetiveCaseOn2 = true
	_, err := GetMysqlDAO().GetData(selectstmt, 25)
	errormdl.IsTestingNegetiveCaseOn2 = false
	assert.Error(t, err, "This should not return error")
}
func TestGetDataError3(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	selectstmt := "SELECT name,age FROM person1 WHERE age =?"
	errormdl.IsTestingNegetiveCaseOn3 = true
	_, err := GetMysqlDAO().GetData(selectstmt, 25)
	errormdl.IsTestingNegetiveCaseOn3 = false
	assert.Error(t, err, "This should not return error")
}
func TestMysqlTransaction(t *testing.T) {
	Init("mysql-config.toml", "localhost")
	connection, connectionError := GetMYSQLConnection("localhost")
	if errormdl.CheckErr(connectionError) != nil {
		loggermdl.LogError(connectionError)
	}
	session := connection.NewSession(nil)
	tx, err := session.Begin()
	if err != nil {
		loggermdl.LogError(err)
	}
	defer tx.RollbackUnlessCommitted()
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	stmt1 := session.InsertBySql(insertstmt1, "mayuri3", 25, true, 2000.00)
	buf := dbr.NewBuffer()
	stmt1.Build(dialect.MySQL, buf)
	query1, interpolateErr1 := dbr.InterpolateForDialect(buf.String(), buf.Value(), dialect.MySQL)
	if interpolateErr1 != nil {
		fmt.Println(interpolateErr1)
	} else {
		fmt.Println(query1) // INSERT INTO `test_upsert` (`id`,`name`,`address`) VALUES ('g','Taro','Tokyo') ON DUPLICATE KEY UPDATE name = 'Taro', address = 'Tokyo'
	}
	if result1, insertErr1 := session.InsertBySql(query1).Exec(); insertErr1 != nil {
		fmt.Println(err)
	} else {
		fmt.Println(result1.RowsAffected())
	}
	insertstmt2 := "INSERT person1 SET Name=?,age=?,isAlive=?,salary=?"
	stmt2 := session.InsertBySql(insertstmt2, "mayuri2", 25, true, 2000.00)
	buf2 := dbr.NewBuffer()
	stmt2.Build(dialect.MySQL, buf2)
	query2, interpolateErr2 := dbr.InterpolateForDialect(buf2.String(), buf2.Value(), dialect.MySQL)
	if interpolateErr2 != nil {
		fmt.Println(interpolateErr2)
	} else {
		fmt.Println(query2) // INSERT INTO `test_upsert` (`id`,`name`,`address`) VALUES ('g','Taro','Tokyo') ON DUPLICATE KEY UPDATE name = 'Taro', address = 'Tokyo'
	}
	if result2, insertErr2 := session.InsertBySql(query2).Exec(); insertErr2 != nil {
		fmt.Println(err)
	} else {
		fmt.Println(result2.RowsAffected())
	}
	tx.Commit()
	assert.NoError(t, err, "This should not return error")
}

func BenchmarkSave(b *testing.B) {
	Init("mysql-config.toml", "localhost")
	insertstmt1 := "INSERT INTO person1 (Name,age,isAlive,salary) VALUES(?,?,?,?)"
	for i := 0; i < b.N; i++ {
		GetMysqlDAOWithHost("localhost").SaveUpdateOrDeleteData(insertstmt1, "mayuri2", 25, true, 2000.00)
	}
}

func BenchmarkUpdate(b *testing.B) {
	Init("mysql-config.toml", "localhost")
	updatestmt := "UPDATE person1 SET isAlive=?,salary=? WHERE Name = ? AND age =?"
	for i := 0; i < b.N; i++ {
		GetMysqlDAOWithHost("localhost").SaveUpdateOrDeleteData(updatestmt, false, 2000.00, "mayuri2", 25)
	}
}
