package filepack

import (
	"fmt"
	"log"
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/filemdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"

	"github.com/tidwall/gjson"
	"github.com/tidwall/sjson"
)

func TestSaveDataInPackFile(t *testing.T) {
	// get file list
	data, _ := sjson.Set("", "name", "vijay")
	data, _ = sjson.Set(data, "studentId", 10014)
	data, _ = sjson.Set(data, "class", "SY_MCA")
	data, _ = sjson.Set(data, "age", 23)
	data, _ = sjson.Set(data, "fileType", "Profile")
	studentObj := gjson.Parse(data)
	IndexFields := []InFileIndexField{
		InFileIndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	// create pack
	err := AddFileInPackFile("/home/vivekn/fdb_data/packFileTest/pack", IndexFields, &studentObj)
	if err != nil {
		log.Fatal(err)
	}
}

func TestGetDataInPackFile(t *testing.T) {

	infileIndexQueries := []string{`#[fileType=="Profile"]`, `#[name=="vijay"]`}
	res, err := GetDataFromPackFile("/home/vivekn/fdb_data/packFileTest/pack1", infileIndexQueries)
	if err != nil {
		log.Fatal(err)
	}
	fmt.Println("res", res)
}

func TestUpdateFileInPackFile(t *testing.T) {
	data, _ := sjson.Set("", "name", "vijay")
	data, _ = sjson.Set(data, "studentId", 10014)
	data, _ = sjson.Set(data, "class", "TY_MCA")
	data, _ = sjson.Set(data, "age", 23)
	data, _ = sjson.Set(data, "fileType", "Profile")
	studentObj := gjson.Parse(data)
	infileIndexQueries := []string{`#[fileType=="Profile"]`, `#[name=="vijay"]`}
	filePaths := []string{`/home/vivekn/fdb_data/packFileTest/pack`, `/home/vivekn/fdb_data/packFileTest/pack1`}
	for _, filePath := range filePaths {
		res, err := UpdateFilesInPackFile(filePath, infileIndexQueries, &studentObj)
		if err != nil {
			log.Fatal(err)
		}
		fmt.Println("res", res)
	}
}

func TestUpdateFileInMultiplePackFile(t *testing.T) {
	data, _ := sjson.Set("", "name", "vijay")
	data, _ = sjson.Set(data, "studentId", 10014)
	data, _ = sjson.Set(data, "class", "TY_MCA")
	data, _ = sjson.Set(data, "age", 23)
	data, _ = sjson.Set(data, "fileType", "Profile")
	studentObj := gjson.Parse(data)
	infileIndexQueries := []string{`#[fileType=="Profile"]`, `#[name=="vijay"]`}
	res, err := UpdateFilesInPackFile("/home/vivekn/fdb_data/packFileTest/pack", infileIndexQueries, &studentObj)
	if err != nil {
		log.Fatal(err)
	}
	fmt.Println("res", res)
}

func TestDeletDataFromPackFile(t *testing.T) {
	infileIndexQueries := []string{`#[fileType=="Profile"]`, `#[name=="vijay"]`}
	_, err := DeletDataFromPackFile("/home/vivekn/fdb_data/packFileTest/pack1", infileIndexQueries)
	if err != nil {
		log.Fatal(err)
	}
}

func TestAddMediaInPackFile(t *testing.T) {
	data, _ := sjson.Set("", "extension", "png")
	data, _ = sjson.Set(data, "fileType", "Asset")
	rs := gjson.Parse(data)
	filePath := "/home/vivekn/Pictures/abc.png"
	dataByte, err := filemdl.ReadFile(filePath)
	if err != nil {
		log.Fatal(err)
	}
	// dataByte = []byte("hello")
	IndexFields := []InFileIndexField{}
	id, err := AddMediaInPackFile("/home/vivekn/fdb_data/packFileTest/pack1", dataByte, IndexFields, &rs)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogError(id)
}
func TestGetMediaFromPackFile(t *testing.T) {
	// data, _ := sjson.Set("", "extension", "png")
	// data, _ = sjson.Set(data, "recordID", "1V6my0ijyVn8Yu5cubfgcUwRiDk")
	// rs := gjson.Parse(data)
	infileIndeQueries := []string{`#[recordID="1V8rdwCsRFyUq91byPE2ad0rb4e"]`}
	filePath := "/home/vivekn/fdb_data/packFileTest/pack1"
	data, err := GetMediaFromPackFile(filePath, infileIndeQueries)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogError(string(data))
}
