package filemdl

import (
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"github.com/stretchr/testify/assert"
)

func TestZipWithoutBaseDirectory(t *testing.T) {
	err := ZipWithoutBaseDirectory("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/TestZipWithoutBaseDirectory.zip")
	assert.NoError(t, err, "This should not throw error")
}

func Test1ZipWithoutBaseDirectory(t *testing.T) {
	// test case for target zip can't be created
	err := ZipWithoutBaseDirectory("../testingdata/testData/filezip/vs-code-settings.json", "X:/Test1ZipWithoutBaseDirectory.zip")
	assert.Error(t, err, "This should throw error")
}

func Test2ZipWithoutBaseDirectory(t *testing.T) {
	// test case for source folder can't be read
	err := ZipWithoutBaseDirectory("X:/vs-code-settings.json", "X:/Test2ZipWithoutBaseDirectory.zip")
	assert.Error(t, err, "This should throw error")
}

func Test3ZipWithoutBaseDirectory(t *testing.T) {
	// test case for walk for source folder
	errormdl.IsTestingNegetiveCaseOn = true
	err := ZipWithoutBaseDirectory("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/Test3ZipWithoutBaseDirectory.zip")
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should throw error")
}

func Test4ZipWithoutBaseDirectory(t *testing.T) {
	// test case for walk for source folder
	err := ZipWithoutBaseDirectory("../testingdata/testData/filezip", "../testingdata/testData/filezip/Test4ZipWithoutBaseDirectory.zip")
	assert.NoError(t, err, "This should not throw error")
}

func TestZipWithSkipFileList(t *testing.T) {
	err := ZipWithSkipFileList("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/TestZipWithSkipFileList.zip", []string{})
	assert.NoError(t, err, "This should not throw error")
}

func Test1ZipWithSkipFileList(t *testing.T) {
	// test case for target zip can't be created
	err := ZipWithSkipFileList("../testingdata/testData/filezip/vs-code-settings.json", "X:/Test1ZipWithSkipFileList.zip", []string{})
	assert.Error(t, err, "This should throw error")
}

func Test2ZipWithSkipFileList(t *testing.T) {
	// test case for source folder can't be read
	err := ZipWithSkipFileList("X:/vs-code-settings.json", "X:/Test2ZipWithSkipFileList.zip", []string{})
	assert.Error(t, err, "This should throw error")
}

func Test3ZipWithSkipFileList(t *testing.T) {
	// test case for walk for source folder
	errormdl.IsTestingNegetiveCaseOn = true
	err := ZipWithSkipFileList("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/Test3ZipWithSkipFileList.zip", []string{})
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should throw error")
}

func Test4ZipWithSkipFileList(t *testing.T) {
	err := Zip("../testingdata/testData/filezip", "../testingdata/testData/Test4ZipWithSkipFileList.zip")
	assert.NoError(t, err, "This should not throw error")
}

func Test5ZipWithSkipFileList(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn = true
	err := Zip("../testingdata/testData/filezip", "../testingdata/testData/Test5ZipWithSkipFileList.zip")
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should not throw error")
}

func Test_checkPathInSkipList(t *testing.T) {
	retVal := checkPathInSkipList("a", []string{"a", "b", "c"})
	assert.True(t, retVal, "This should return true")
}
func Test1_checkPathInSkipList(t *testing.T) {
	retVal := checkPathInSkipList("d", []string{"a", "b", "c"})
	assert.False(t, retVal, "This should return false")
}
func Test_Unzip(t *testing.T) {
	Zip("../testingdata/testData/filezip", "../testingdata/testData/Test_Unzip.zip")
	retVal := Unzip("../testingdata/testData/Test_Unzip.zip", "../testingdata/testData/filezip/Test_Unzip")
	assert.NoError(t, retVal, "This should not return error")
}
func Test1_Unzip(t *testing.T) {
	Zip("../testingdata/testData/filezip", "../testingdata/testData/Test1_Unzip.zip")
	retVal := Unzip("X:/Test1_Unzip.zip", "../testingdata/testData/filezip/Test1_Unzip")
	assert.Error(t, retVal, "This should return error")
}
func Test2_Unzip(t *testing.T) {
	Zip("../testingdata/testData/filezip", "../testingdata/testData/Test2_Unzip.zip")
	retVal := Unzip("../testingdata/testData/Test2_Unzip.zip", "X:/Test2_Unzip")
	assert.Error(t, retVal, "This should return error")
}

func Test3_Unzip(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn = true
	Zip("../testingdata/testData/filezip", "../testingdata/testData/Test3_Unzip.zip")
	retVal := Unzip("../testingdata/testData/Test3_Unzip.zip", "../testingdata/testData/filezip/Test3_Unzip")
	assert.Error(t, retVal, "This should return error")
	errormdl.IsTestingNegetiveCaseOn = false
}
