package filemdl

import (
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"github.com/stretchr/testify/assert"
)

func TestZipWithFileListAndSkips(t *testing.T) {
	filePathList := []string{"../testingdata/testData/testFile1", "../testingdata/testData/testFile2", "../testingdata/testData/testFile3", "../testingdata/testData/testFiles"}
	skipFileList := []string{"testFile6"}
	err := ZipWithFileListAndSkips(filePathList, "../testingdata/testData/filezip/TestZipWithFileListAndSkips.zip", skipFileList)
	assert.NoError(t, err, "This should not throw error")
}
func Test1ZipWithFileListAndSkips(t *testing.T) {
	// test case for target zip can't be created
	filePathList := []string{"../testingdata/testData/testFile1", "../testingdata/testData/testFile2", "../testingdata/testData/testFile3", "../testingdata/testData/testFiles"}
	skipFileList := []string{"../testingdata/testData/testFiles"}

	err := ZipWithFileListAndSkips(filePathList, "X:/Test1ZipWithFileListAndSkips.zip", skipFileList)
	assert.Error(t, err, "This should throw error")
}
func Test2ZipWithFileListAndSkips(t *testing.T) {
	// test case for source folder can't be read
	filePathList := []string{"X:/vs-code-settings.json", "../testingdata/testData/testFile2", "../testingdata/testData/testFile3", "../testingdata/testData/testFiles"}
	skipFileList := []string{"../testingdata/testData/testFiles"}
	err := ZipWithFileListAndSkips(filePathList, "../testingdata/testData/filezip/Test2ZipWithFileListAndSkips.zip", skipFileList)
	assert.Error(t, err, "This should throw error")
}

func TestZipWithFileList(t *testing.T) {
	filePathList := []string{"../testingdata/testData/testFile1", "../testingdata/testData/testFile2", "../testingdata/testData/testFile3", "../testingdata/testData/testFiles"}
	err := ZipWithFileList(filePathList, "../testingdata/testData/filezip/TestZipWithFileList.zip")
	assert.NoError(t, err, "This should not throw error")
}
func Test1ZipWithFileList(t *testing.T) {
	// test case for target zip can't be created
	filePathList := []string{"../testingdata/testData/testFile1", "../testingdata/testData/testFile2", "../testingdata/testData/testFile3", "../testingdata/testData/testFiles"}

	err := ZipWithFileList(filePathList, "X:/Test1ZipWithFileList.zip")
	assert.Error(t, err, "This should throw error")
}
func Test2ZipWithFileList(t *testing.T) {
	// test case for source folder can't be read
	filePathList := []string{"X:/vs-code-settings.json", "../testingdata/testData/testFile2", "../testingdata/testData/testFile3", "../testingdata/testData/testFiles"}
	err := ZipWithFileList(filePathList, "../testingdata/testData/filezip/Test2ZipWithFileList.zip")
	assert.Error(t, err, "This should throw error")
}
func TestZip(t *testing.T) {
	err := Zip("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/TestZip.zip")
	assert.NoError(t, err, "This should not throw error")
}

func Test1Zip(t *testing.T) {
	// test case for target zip can't be created
	err := Zip("../testingdata/testData/filezip/vs-code-settings.json", "X:/Test1Zip.zip")
	assert.Error(t, err, "This should throw error")
}

func Test2Zip(t *testing.T) {
	// test case for source folder can't be read
	err := Zip("X:/vs-code-settings.json", "X:/Test2Zip.zip")
	assert.Error(t, err, "This should throw error")
}

func Test3Zip(t *testing.T) {
	// test case for walk for source folder
	errormdl.IsTestingNegetiveCaseOn = true
	errormdl.IsTestingNegetiveCaseOn1 = true
	err := Zip("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/Test3Zip.zip")
	errormdl.IsTestingNegetiveCaseOn = false
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should throw error")
}

func Test4Zip(t *testing.T) {
	err := Zip("../testingdata/testData/filezip", "../testingdata/testData/filezip/Test4Zip.zip")
	assert.NoError(t, err, "This should not throw error")
}

func Test5Zip(t *testing.T) {
	err := Zip("../testingdata/testData/filezip", "../testingdata/testData/filezip/Test5Zip.zip")
	assert.NoError(t, err, "This should not throw error")
}

func Test_ZipBytes(t *testing.T) {
	inputData := []byte{1, 2, 3, 4, 5}
	_, err := ZipBytes(inputData)
	assert.NoError(t, err, "This should not throw error")
}

func Test1_ZipBytes(t *testing.T) {
	inputData := []byte{}
	_, err := ZipBytes(inputData)
	assert.Error(t, err, "This should throw error")
}

func TestZipSingleFile(t *testing.T) {
	err := ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/TestZipSingleFile.zip")
	assert.NoError(t, err, "This should not throw error")
}

func Test1ZipSingleFile(t *testing.T) {
	err := ZipSingleFile("X:/vs-code-settings.json", "../testingdata/testData/filezip/Test1ZipSingleFile.zip")
	assert.Error(t, err, "This should throw error")
}

func Test2ZipSingleFile(t *testing.T) {
	err := ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "X:/Test1ZipSingleFile.zip")
	assert.Error(t, err, "This should throw error")
}

func Test3ZipSingleFile(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn = true
	err := ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/Test2ZipSingleFile.zip")
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should not throw error")
}
func Test_UnZipBytes(t *testing.T) {
	inputData := []byte{1, 2, 3, 4, 5}
	inputData, _ = ZipBytes(inputData)
	_, err := UnZipBytes(inputData)
	assert.NoError(t, err, "This should not throw error")
}

func Test1_UnZipBytes(t *testing.T) {
	inputData := []byte{1, 2, 3, 4, 5}
	_, err := UnZipBytes(inputData)
	assert.Error(t, err, "This should throw error")
}

func TestUnZipSingleFile(t *testing.T) {
	ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/TestUnZipSingleFile.zip")
	err := UnZipSingleFile("../testingdata/testData/filezip/TestUnZipSingleFile.zip", "../testingdata/testData/filezip/uz-vs-code-settings.json")
	// fmt.Println("err: ", err)
	assert.NoError(t, err, "This should not throw error")
}

func Test1UnZipSingleFile(t *testing.T) {
	ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/Test1UnZipSingleFile.zip")
	err := UnZipSingleFile("X:/TestUnZipSingleFile.zip", "../testingdata/testData/filezip/uz-vs-code-settings.json")
	// fmt.Println("err: ", err)
	assert.Error(t, err, "This should throw error")
}

func Test2UnZipSingleFile(t *testing.T) {
	ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/Test2UnZipSingleFile.zip")
	err := UnZipSingleFile("../testingdata/testData/filezip/Test2UnZipSingleFile.zip", "X:/uz-vs-code-settings.json")
	// fmt.Println("err: ", err)
	assert.Error(t, err, "This should throw error")
}

func Test3UnZipSingleFile(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn = true
	ZipSingleFile("../testingdata/testData/filezip/vs-code-settings.json", "../testingdata/testData/filezip/TestUnZipSingleFile.zip")
	err := UnZipSingleFile("../testingdata/testData/filezip/TestUnZipSingleFile.zip", "../testingdata/testData/filezip/uz-vs-code-settings.json")
	// fmt.Println("err: ", err)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.Error(t, err, "This should not throw error")
}
