package email

import (
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"github.com/stretchr/testify/assert"
)

type Person struct {
	FirstName string
	LastName  string
	Emails    []string
}

func TestInit(t *testing.T) {
	errormdl.IsTestingNegetiveCaseOn = true
	err := Init("email-config.toml")
	errormdl.IsTestingNegetiveCaseOn = false
	assert.NoError(t, err, "This should not return error")
}
func TestSendMail(t *testing.T) {
	persondata := Person{
		FirstName: "Mayuri",
		LastName:  "Shinde",
		Emails:    []string{"mayuris@gmail.com", "mayuri@gmail.com"},
	}
	from := "mayuris@mkcl.org"
	subject := "Test Email gomail"
	to := []string{"mayuri92shinde@gmail.com"}
	// cc := []string{"onkarh@mkcl.org", "kumargauravs@mkcl.org"}
	// bcc := []string{"Prajkty@mkcl.org", "rakeshd@mkcl.org"}
	//attachments := []string{"go.jpg"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	err := mail.SendMail("email-template.html", persondata)
	assert.NoError(t, err, "This should not return error")
}

func TestSendMailErrorWrongTemplt(t *testing.T) {
	persondata := Person{
		FirstName: "Mayuri",
		LastName:  "Shinde",
		Emails:    []string{"mayuris@gmail.com", "mayuri@gmail.com"},
	}
	from := "mayuris@mkcl.org"
	subject := "Test Email gomail"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	err := mail.SendMail("wrong.html", persondata)
	assert.Error(t, err, "This should return error")
}

func TestSendMailError(t *testing.T) {
	persondata := Person{
		FirstName: "Mayuri",
		LastName:  "Shinde",
		Emails:    []string{"mayuris@gmail.com", "mayuri@gmail.com"},
	}
	from := "mayuris@mkcl.org"
	subject := "Test Email gomail"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	errormdl.IsTestingNegetiveCaseOn1 = true
	err := mail.SendMail("email-template.html", persondata)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should return error")
}

func TestSendMailError1(t *testing.T) {
	persondata := Person{
		FirstName: "Mayuri",
		LastName:  "Shinde",
		Emails:    []string{"mayuris@gmail.com", "mayuri@gmail.com"},
	}
	from := "mayuris@mkcl.org"
	subject := "Test Email gomail"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	errormdl.IsTestingNegetiveCaseOn1 = true
	err := mail.SendMail("email-template.html", persondata)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should return error")
}

func BenchmarkSendMail(b *testing.B) {
	persondata := Person{
		FirstName: "Mayuri",
		LastName:  "Shinde",
		Emails:    []string{"mayuris@gmail.com", "mayuri@gmail.com"},
	}
	from := "mayuris@mkcl.org"
	subject := "Test Email smtp"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	for i := 0; i < b.N; i++ {
		mail.SendMail("email-template.html", persondata)
	}
}
func TestSendMailWithHandlebar(t *testing.T) {
	tpl1 := `<div class="post">
	<h1>By {{author.firstName}} {{author.lastName}}</h1>
	<div class="body">{{body}}</div>
  
	<h1>Comments</h1>
  
	{{#each comments}}
	<h2>By {{author.firstName}} {{author.lastName}}</h2>
	<div class="body">{{content}}</div>
	{{/each}}
  </div>`

	type Person struct {
		FirstName string
		LastName  string
	}

	type Comment struct {
		Author Person
		Body   string `handlebars:"content"`
	}

	type Post struct {
		Author   Person
		Body     string
		Comments []Comment
	}

	ctx1 := Post{
		Person{"Jean", "Valjean"},
		"Life is difficult",
		[]Comment{
			Comment{
				Person{"Marcel", "Beliveau"},
				"LOL!",
			},
		},
	}
	from := "mayuris@mkcl.org"
	subject := "Test SendMailWithHandlebar"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	err := mail.SendMailWithHandlebar(tpl1, ctx1)
	assert.NoError(t, err, "This should not return error")
}
func TestSendMailWithHandlebarError(t *testing.T) {
	tpl2 := `<div class="entry">
	<h1>{{title}}</h1>
	<div class="body">
	  {{body}}
	</div>
  </div>
  `
	ctx2 := []map[string]string{
		{
			"title": "My New Post",
			"body":  "This is my first post!",
		},
		{
			"title": "Here is another post",
			"body":  "This is my second post!",
		},
	}
	from := "mayuris@mkcl.org"
	subject := "Test SendMailWithHandlebarError"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	errormdl.IsTestingNegetiveCaseOn = true
	err := mail.SendMailWithHandlebar(tpl2, ctx2)
	errormdl.IsTestingNegetiveCaseOn = false
	assert.NoError(t, err, "This should not return error")
}

func TestSendMailWithHandlebarError1(t *testing.T) {
	tpl1 := `<div class="entry">
	<h1>{{title}}</h1>
	<div class="body">
	  {{body}}
	</div>
  </div>
  `

	ctx1 := map[string]string{
		"title": "My New Post",
		"body":  "This is my first post!",
	}
	from := "mayuris@mkcl.org"
	subject := "Test SendMailWithHandlebarError1"
	to := []string{"mayuri92shinde@gmail.com"}
	Init("email-config.toml")
	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
	errormdl.IsTestingNegetiveCaseOn1 = true
	err := mail.SendMailWithHandlebar(tpl1, ctx1)
	errormdl.IsTestingNegetiveCaseOn1 = false
	assert.Error(t, err, "This should return error")
}

// func BenchmarkSendMailSMTP(b *testing.B) {
// 	persondata := Person{
// 		FirstName: "Mayuri",
// 		LastName:  "Shinde",
// 		Emails:    []string{"mayuris@gmail.com", "mayuri@gmail.com"},
// 	}
// 	from := "mayuris@mkcl.org"
// 	subject := "Test Email smtp"
// 	to := []string{"mayuri92shinde@gmail.com"}
// 	Init("email-config.toml")
// 	mail := NewEmail(to, []string{}, []string{}, []string{}, from, subject, "")
// 	for i := 0; i < b.N; i++ {
// 		mail.SendMailSMTP("email-template.html", persondata)
// 	}
// }
